\NeedsTeXFormat{LaTeX2e}
\ProvidesClass{scrskoma}[2024/03/05 Based on KOMA, loads many other packages]
% The date format should be YYYY/MM/DD :(
% This class originated partially from the Daedalus 2 SED and then my (Philip's) master thesis.
% I do wonder how long it will live on somewhere... Hello to you, future creature :)


%-------------------------------------------------------------------------------
% Short usage information:
% - Specify "superclass" to use with documentclass option
%   scls=scrartcl|article|scrreprt|report|scrbook|book
% - Use final as document class option to suppress the various \todo[...]
% - Babel language goes into document options, last language is the main one
%-------------------------------------------------------------------------------


%-------------------------------------------------------------------------------
%   Options, Parameters and Superclass
%-------------------------------------------------------------------------------

% Options defining the superclass
% Yes, I am aware that keyval packages exist, but I tired my luck with
% kvoptions and it did not work because to option processing order
% was wrong (i.e. the super call macro still had the default option when
% processing the undeclared options), so I gave up

\newcommand{\scrskoma@scls}{scrartcl}

% article / scrarticle
% - typically for shorter documents documents
% - hightest sectioning level is \section
\DeclareOption{scls=scrartcl}{
    \renewcommand{\scrskoma@scls}{scrartcl}
}
\DeclareOption{scls=article}{
    \renewcommand{\scrskoma@scls}{scrartcl}
}

% report / scrreprt
% - for longer articles, shorter books or thesis
% - hightest sectioning level is \chaption
\DeclareOption{scls=scrreprt}{
    \renewcommand{\scrskoma@scls}{scrreprt}
}
\DeclareOption{scls=report}{
    \renewcommand{\scrskoma@scls}{scrreprt}
}

% book / scrbook
% - hightest sectioning level is \part
% - has \frontmatter, \mainmatter, \backmatter commands
% - no built-in abstract environment
\DeclareOption{scls=scrbook}{
    \renewcommand{\scrskoma@scls}{scrbook}
}
\DeclareOption{scls=book}{
    \renewcommand{\scrskoma@scls}{scrbook}
}

% All unknown parameters get passed to the superclass
% I am not sure why this is needed b/c the documentclass options
% are global options and therefore should also be seen by the superclass.
% But it is required
\DeclareOption*{
    \PassOptionsToClass{\CurrentOption}{\scrskoma@scls}
}

% Process options in the order they appear
% (w/o star options are processed in the order declared in the package file)
% https://tex.stackexchange.com/a/82441
\ProcessOptions*

% Default options for superclasses could be specified here with
%\PassOptionsToClass{options}{class}
% these would only get applied when that class is then later actually loaded

% load superclass
\LoadClass{\scrskoma@scls}

% load scrhack which fixes some other packages to work better with KOMAScript
\RequirePackage{scrhack}


%-------------------------------------------------------------------------------
%   Language
%-------------------------------------------------------------------------------

% Language should come from the global document class options
% Last language in document class options is the main one
\RequirePackage{babel}


%-------------------------------------------------------------------------------
%   General layout
%-------------------------------------------------------------------------------

% Include list of figures and tables in TOC
\KOMAoptions{listof=totoc, listof=flat}

% Include everything down to subsubsections in the TOC
\setcounter{tocdepth}{\subsubsectiontocdepth}

% and number everything down to subsubsections
\setcounter{secnumdepth}{\subsubsectionnumdepth}

% Print chapter titles in the header
\RequirePackage[
    % automatic page titles from the chapter name
    automark,
    % print a line separating the header from the rest of the text
    headsepline
]{scrlayer-scrpage}

% Not required, scrheadings replaces the default headings and plain page styles
%\pagestyle{scrheadings}


%-------------------------------------------------------------------------------
%   Math
%-------------------------------------------------------------------------------

% \DeclareMathOperator, also required by commath, so load explicitly
\RequirePackage{amsmath}

% For \mathbb (e.g. natural numbers symbol)
\RequirePackage{amssymb}

% commath for \dif (dt in integrals) among others
\RequirePackage{commath}

% \bm for bold italic letters in math
\RequirePackage{bm}

% New math operators
% arctan2(y, x)
\DeclareMathOperator{\arctantwo}{arctan2}

% Shorthands
\newcommand{\logx}{\log_{10}}
\newcommand{\mbeq}{\overset{!}{=}}  % "mbeq": must be equal


%-------------------------------------------------------------------------------
%   SI Unit Formatting
%-------------------------------------------------------------------------------

% Enables use of \SI{number}[pre-unit]{unit}
\RequirePackage{siunitx}
%\sisetup{locale = US}  % locale only, language comes from babel
%\sisetup{per-mode=symbol, per-symbol = p} % for bits per second

\DeclareSIUnit\gauss{G}
\DeclareSIUnit\dBW{dbW}
\DeclareSIUnit\dBm{dBm}


%-------------------------------------------------------------------------------
%   General Packages without special config
%-------------------------------------------------------------------------------

% Include and input packages from subfolders with relative input
\RequirePackage{import}

% Request pages (and their content) to be in landscape mode by placing them inside
% \begin{landscape}  [... content ...]  \end{landscape}
% With scrhack \textheight and \textwidth keep their original values,
% without scrhack \textheight becomes \textwidth, but \textwidth retains
% it's original value (which is inconsistent and confusing).
\RequirePackage{pdflscape}


%-------------------------------------------------------------------------------
%   Fonts and Typesetting
%-------------------------------------------------------------------------------

% Makes more beautiful text
\RequirePackage{microtype}

% Smart, context sensitive quotes
% Active Quotes are not enabled, so you will always want to use \enquote and
% friends
% After: babel (so that autostyle works)
\RequirePackage[autostyle]{csquotes}


%-------------------------------------------------------------------------------
%   Colors
%-------------------------------------------------------------------------------

% required at least for the linkcolor option to hyperref
\RequirePackage{xcolor}


%-------------------------------------------------------------------------------
%   Floats, Figures and Tables, common stuff
%-------------------------------------------------------------------------------

% KOMA option: no hanging indents for captions
\setcapindent{0pt}

% Provides (among probably other things) the capital H placement modifier
% Causes the `\float@addtolists detected!` warning from KOMAScript when
% used without loading scrhack.
% With scrhack it uses KOMA's tocbasic (more infos in scrguide.pdf)
\RequirePackage{float}

% For figures and tables that are placed "on the side"
\RequirePackage{wrapfig}

% Provides subfigure and subtable environments
% Replacement for the deprecated subfigure
\RequirePackage{subcaption}


%-------------------------------------------------------------------------------
%   Graphics / Figures
%-------------------------------------------------------------------------------

% For graphics
% Gets later loaded by todonotes anyway, but I figure its good style to load
% it explicitly here.
\RequirePackage{graphicx}

% Fancy command-defined graphics
\RequirePackage{tikz}

% Required for "valign=m, margin=.5em" in \includegraphics
% Basically, this makes it so pictures in tables work as 'floats'
% and not just one big character that makes the line as tall as itself.
\RequirePackage[export]{adjustbox}


%-------------------------------------------------------------------------------
%   Tables
%-------------------------------------------------------------------------------

% For fancier tables & \toprule, \midrule and \bottomrule
\RequirePackage{booktabs}

% For tables that are as wide as a page
\RequirePackage{tabularx}


%-------------------------------------------------------------------------------
%   hyperref package and PDF metadata
%-------------------------------------------------------------------------------

% Pass hypens option to url, when it later gets loaded by hyperref
% This is required so the line breaks in URLs work properly
\PassOptionsToPackage{hyphens}{url}

\RequirePackage[% Explanations are copy&paste from the hyperref doc
    pdfpagemode=UseOutlines, % Determines how the file is opening in Acrobat
    pdfdisplaydoctitle=true, % display document title instead of file name in title bar
    pdfpagelayout=OneColumn, % set layout of PDF pages
    hypertexnames=false, % do not use guessable names for links
    % pdfusetitle breaks with \and in the authors list
    %pdfusetitle=true, % use \title and \author for PDF metadata
    % Bookmark (this is the table of conents displayed in your PDF viewer) config
    bookmarks=true,
    bookmarksopen=true,
    bookmarksopenlevel=2, % levels of bookmarks to expand, not sure what the number exactly means...
    bookmarksnumbered=true,
    colorlinks=true,
    linkcolor={blue!50!black},
    citecolor={green!50!black},
    urlcolor={magenta!50!black}
  ]{hyperref}

% Command to override automatic author extraction from \author
% because the automatic extraction will fail quite easily
% \@safepdfauthor b/c \@pdfauthor is already used by hyperref
\newcommand{\@safepdfauthor}{}
\def\@safepdfauthor{}  % required or the \ifx\@safepdfauthor\empty is false
\newcommand{\pdfauthor}[1]{\def\@safepdfauthor{#1}}

% Force-Feed hyperref the subject, also custom title
% This needs to happen AtBeginDocument b/c only than are all the @vars
% correctly defined
\AtBeginDocument{
    \makeatletter%
    \begingroup% to protentially scope the \def\and
    % automatic author extraction if \@safepdfauthor is still empty
    % Note: if no \author has ever been set, \@author is
    % \@latex@warning@no@line {No \noexpand \author given}
    % which, when trying to write it to PDF document metadata
    % produces strange warnings from hyperref and also an author that is just
    % a bunch of chinese characters
    \ifx\@safepdfauthor\empty%
        % make \and work in the pdf author list
        % replace the \and by "and " as is standard in biblatex author lists
        % (in \author you must have {a \and b \and c}, so a space before and)
        \def\and{and }%
        % \edef means expand completely what is in {}
        % so replace \and by what we defined it above to be
        \edef\@safepdfauthor{\@author}%
    \else\fi%
    \hypersetup{
        pdfsubject={\@subject},
        pdftitle={\@title},
        pdfauthor={\@safepdfauthor}
    }%
    \endgroup%
    \makeatother%
}

% Add table of contents to pdf bookmarks
\BeforeTOCHead[toc]{%
    \if@openright\cleardoublepage\else\clearpage\fi%
    \pdfbookmark[0]{\contentsname}{toc}%
}

% Util command
\newcommand{\email}[1]{\href{mailto:#1}{#1}}


%-------------------------------------------------------------------------------
%   Bibliography
%-------------------------------------------------------------------------------

% BibLaTeX is responsible for all the bibliography stuff
\RequirePackage[
    backend=biber, % unicode support with biber!
    %style=alphabetic, % style in bibliograpgy & citations. Format: [Phi99]
    style=numeric, % style in bibliograpgy & citations. Format: [1]
    maxnames=1000, % give credit to at most 1000 names before resorting to "Author1 et Al"
    natbib=true, % provide natbib compatability
    sorting=nyt, % sort by name, year, title
    autolang=other % cite an entry in the language of its langid
]{biblatex}


%-------------------------------------------------------------------------------
%   Glossary and Acronyms
%-------------------------------------------------------------------------------

% hyperref, babel, polyglossia, inputenc and fontenc would need to be loaded
% first, if they are used
\RequirePackage[
    acronym, % create acronym glossary
    nonumberlist, % no page numbers
    toc, % put in toc
    sort=def % do not sort, just use the entries in the order they appear
]{glossaries}
% TODO: Sorting with external tools and documentation


%-------------------------------------------------------------------------------
%   TODO Notes
%-------------------------------------------------------------------------------

% Todo notes (\todo{Todo note here})
\RequirePackage[obeyFinal, colorinlistoftodos, textsize=tiny]{todonotes}
%\presetkeys{todonotes}{inline}{}

% Create todo page at document begin if todos are enabled
% If final option is given, nothing is printed for \listoftodos,
% so \newpage does nothing
% Note: \listoftodos dies in twocolumn mode
\AtBeginDocument{
    \setcounter{page}{-100}
    \todototoc
    \listoftodos
    % The following _should_ normally be handled by \maketitle
    % But at least the page counter reset does not happen with scrartcl :/
    \newpage
    \setcounter{page}{1}
}

% Some help macros for todos that alrady classify the problem
% #1 = Parameter for todo note (optional arg)
% #2 = Content of todo note

% In case content is missing, inline by default
\newcommand{\todoContent}[2][]{\todo[color=pink!40, inline, #1]{#2 (Content missing)}}

% In case there is an (suspected) error
\newcommand{\todoError}[2][]{\todo[color=red!10, #1]{error: #2}}

% In case things are unsure and should be verified again or the source is
% missing
\newcommand{\todoCN}[2][]{\todo[color=lime!40, #1]{#2 (citation needed)}}
\newcommand{\todoRef}[2][]{\todo[color=lime!40, #1]{#2 (reference missing)}}

% Formatting problems
\newcommand{\todoFormat}[2][]{\todo[color=cyan!40, #1]{#2 (formatting needs fixing)}}

% An editor is required here, inline by default
\newcommand{\todoEditor}[2][]{\todo[color=teal!40, inline, #1]{edit required: #2}}


%-------------------------------------------------------------------------------
%   Misc
%-------------------------------------------------------------------------------

% For the CC icons
\RequirePackage{ccicons}
